const updaterModel = require("../model/updaterModel");
const updaterProdFiliModel = require("../model/updaterProdFiliModel");
const GeraNovoCodigo = require("../modules/NovoCodigoModule");
const OracleService = require("./OracleService");
const Base64 = require("../functions/Base64");
const { v4: uuidv4 } = require("uuid");

class UpdaterService {
  async insertEventTable(cdFilial, cdOperador, devopsId, prodFiliId, currentVersion, installVersion, createNewVersion, action) {
    let response = {
      error: true,
      message: "O evento já foi registrado no sistema.",
    };
   
    let params = {
      P_CDFILIAL: cdFilial,
      P_CDPRODUCTID: devopsId,
    };

    const unitEvents = await updaterModel.getEventsByUnit(params);

    if (unitEvents.rows.length === 0) {
      const eventSeq = await GeraNovoCodigo.prxUpdaterEvento();
      const uuid = uuidv4();

      params = {
        P_CDFILIAL: cdFilial,
        P_CDEVENTO: eventSeq,
        P_CDUUID: uuid,
        P_CDPRODUCTID: devopsId,
        P_CDPRODFILIID: prodFiliId,
        P_IDSTEVENTO: action,
        P_NRVERSAOATUAL: currentVersion,
        P_NRVERSAOALVO: installVersion,
        P_IDCRIARVERSAO: createNewVersion,
      };

      await updaterModel.insertPendingEvent(params);

      const historySeq = await GeraNovoCodigo.prxUpdaterHistorico();

      params = {
        P_CDFILIAL: cdFilial,
        P_CDHISTORICO: historySeq,
        P_CDEVENTO: eventSeq,
        P_CDUUID: uuid,
        P_CDOPERADOR: cdOperador,
        P_CDPRODUCTID: devopsId,
        P_NMPRODUCT: null,
        P_IDSTEVENTO: action,
        P_NRVERSAOATUAL: currentVersion,
        P_NRVERSAOALVO: installVersion,
        P_IDCRIARVERSAO: createNewVersion,
      };

      await updaterModel.appendUpdaterHistory(params);

      response = {
        error: false,
        message: "Evento cadastrado com sucesso.",
      };

      await OracleService.commit();
    }

    return response;
  }

  async updateEventTable(cdFilial, cdOperador, devopsId, currentVersion, installVersion, createNewVersion, action) {
    let response = {
      error: true,
      message: "O evento já foi atualizado no sistema.",
    };

    const eventId = action === "INSTALACAO_EM_CURSO" ? "INSTALACAO_PENDENTE" : "REMOCAO_PENDENTE";

    let params = {
      P_CDFILIAL: cdFilial,
      P_CDPRODUCTID: devopsId,
      P_IDSTEVENTO: eventId,
    };

    const pendingEvents = await updaterModel.getProductUpdaterEventByEventId(params);

    if (pendingEvents.rows.length > 0) {
      const uuid = uuidv4();

      params = {
        P_IDSTEVENTO: action,
        P_CDEVENTO: pendingEvents.rows[0].CDEVENTO,
      };

      await updaterModel.updateEventInProgress(params);

      const history = await updaterModel.getUpdaterHistoryByEventId(params);

      if (history.rows.length === 0) {
        const historySeq = await GeraNovoCodigo.prxUpdaterHistorico();

        params = {
          P_CDFILIAL: cdFilial,
          P_CDHISTORICO: historySeq,
          P_CDEVENTO: pendingEvents.rows[0].CDEVENTO,
          P_CDUUID: uuid,
          P_CDOPERADOR: cdOperador,
          P_CDPRODUCTID: devopsId,
          P_NMPRODUCT: null,
          P_IDSTEVENTO: action,
          P_NRVERSAOATUAL: currentVersion,
          P_NRVERSAOALVO: installVersion,
          P_IDCRIARVERSAO: createNewVersion,
        };

        await updaterModel.appendUpdaterHistory(params);

        response = {
          error: false,
          message: "Evento atualizado com sucesso. Histórico criado",
        };
      }
      await OracleService.commit();
    }

    return response;
  }

  async finishEventTable(cdFilial, cdOperador, devopsId, prodFiliId, currentVersion, installVersion, createNewVersion) {
    let response = {
      error: true,
      message: "O evento já foi atualizado no sistema.",
    };

    let params = {
      P_CDFILIAL: cdFilial,
      P_CDPRODUCTID: devopsId,
    };

    const inProgressEvents = await updaterModel.getInProgressEvents(params);

    if (inProgressEvents.rows.length > 0) {
      const uuid = uuidv4();

      params = {
        P_IDSTEVENTO: "FINALIZADO",
        P_CDEVENTO: inProgressEvents.rows[0].CDEVENTO,
      };

      await updaterModel.updateEventInProgress(params);

      const history = await updaterModel.getUpdaterHistoryByEventId(params);

      if (history.rows.length === 0) {
        const historySeq = await GeraNovoCodigo.prxUpdaterHistorico();

        params = {
          P_CDFILIAL: cdFilial,
          P_CDHISTORICO: historySeq,
          P_CDEVENTO: inProgressEvents.rows[0].CDEVENTO,
          P_CDUUID: uuid,
          P_CDOPERADOR: cdOperador,
          P_CDPRODUCTID: devopsId,
          P_NMPRODUCT: null,
          P_IDSTEVENTO: "FINALIZADO",
          P_NRVERSAOATUAL: currentVersion,
          P_NRVERSAOALVO: installVersion,
          P_IDCRIARVERSAO: createNewVersion,
        };

        await updaterModel.appendUpdaterHistory(params);

        await this.updateProdFiliVersion(prodFiliId, installVersion);

        response = {
          error: false,
          message: "Evento atualizado com sucesso. Histórico criado",
        };
      }

      await OracleService.commit();
    }

    return response;
  }

  async updateProdFiliVersion(prodFiliId, updateVersion) {
    const params = {
      NRVERSAO: updateVersion,
      ID: prodFiliId,
    };

    await updaterProdFiliModel.updateUpdaterProdFiliVersion(params);
  }

  async cancelUnitEvents(cdFilial) {
    let params = {
      P_CDFILIAL: cdFilial,
    };

    await updaterModel.cancelUnitEvents(params);

    await OracleService.commit();
  }

  async cancelEventTable(cdFilial, cdOperador, devopsId, currentVersion, installVersion, createNewVersion) {
    let response = {
      error: true,
      message: "O evento já foi atualizado no sistema.",
    };

    let params = {
      P_CDFILIAL: cdFilial,
      P_CDPRODUCTID: devopsId,
    };

    const unitEvents = await updaterModel.getEventsByUnit(params);

    if (unitEvents.rows.length > 0) {
      const uuid = uuidv4();

      params = {
        P_IDSTEVENTO: "CANCELADO",
        P_CDEVENTO: unitEvents.rows[0].CDEVENTO,
      };

      await updaterModel.updateEventInProgress(params);

      const history = await updaterModel.getUpdaterHistoryByEventId(params);

      if (history.rows.length === 0) {
        const historySeq = await GeraNovoCodigo.prxUpdaterHistorico();

        params = {
          P_CDFILIAL: cdFilial,
          P_CDHISTORICO: historySeq,
          P_CDEVENTO: unitEvents.rows[0].CDEVENTO,
          P_CDUUID: uuid,
          P_CDOPERADOR: cdOperador,
          P_CDPRODUCTID: devopsId,
          P_NMPRODUCT: null,
          P_IDSTEVENTO: "CANCELADO",
          P_NRVERSAOATUAL: currentVersion,
          P_NRVERSAOALVO: installVersion,
          P_IDCRIARVERSAO: createNewVersion,
        };

        await updaterModel.appendUpdaterHistory(params);

        response = {
          error: false,
          message: "Evento atualizado com sucesso. Histórico criado",
        };
      }
      await OracleService.commit();
    }

    return response;
  }

  async getPendingEvents() {
    return updaterModel.getPendingEvents();
  }

  async getPendingEventsByUnit(cdFilial) {
    try {
      const params = {
        P_CDFILIAL: cdFilial,
      };
  
      const pendingEvents = await updaterModel.getPendingEventsByUnit(params);
  
      if (pendingEvents.rows.length === 0) {
        throw 'Nenhum evento pendente de instalação/remoção encontrado.';
      }
      
      return {
        error: false,
        message: 'Nenhum evento pendente de instalação/remoção encontrado.',
        events: pendingEvents.rows
      }
    } catch (error) {
      return {
        error: true,
        message: error,
        events: {}
      }
    }
  }
  
  async checkVersion(params) {
    const result = await updaterModel.getCheckInstalledVersion(params);
    return { error: result.rows.length > 0 };
  }

  async saveConfig({ config }){
    try {
      if (!config) {
        throw {
          error: true,
          message: 'Codigo da Filial não foi informado'
        };
      }

      const params = {
        P_CDFILIAL: config.environment.CDFILIAL
      };
      
      let configIni = await updaterModel.getConfigIni(params);

      if (configIni.rows.length > 0) {
        throw {
          error: false,
          message: `Arquivo config.ini da Filial ${params.P_CDFILIAL} já está salvo na base de dados.`
        };
      };

      const configIniBase64 = Base64.encode(config);         
      
      if (!configIniBase64) {
        throw {
          error: true,
          message: 'Não foi possivel converter config.ini da filial para base64.'
        };
      }
      
      params.P_CONFIG = configIniBase64;

      const insertConfig = await updaterModel.insertConfigIni(params);

      if (insertConfig.rowsAffected === 0) {
        throw {
          error: true,
          message: 'Erro ao inserir as informações do config.ini na base de dados.'
        };
      }

      configIni = await updaterModel.getConfigIni(params);

      if (!configIni) {
        throw {
          error: true,
          message: 'Não foi possivel recuperar as informações do config.ini da filial.'
        };
      }

      const operator = await updaterModel.getDefaultOperator(params);

      if (operator.rows.length === 0) {
        throw {
          error: true,
          message: `Não foi possivel buscar o operador/vendedor padrão da filial ${params.P_CDFILIAL}.`
        };
      }

      if (operator.rows[0].CDOPERADOR === null && operator.rows[0].CDVENDEDOR === null) {
        throw {
          error: true,
          message: `Os campos de Operador Padrão e Vendedor Padrão da filial ${params.P_CDFILIAL} não estão preenchidos.`
        };
      }
      const { CDOPERADOR } = operator.rows[0]; 
      const defaultOperator = CDOPERADOR ? CDOPERADOR : await updaterModel.getDefaultOperatorFromSalesman({P_CDVENDEDOR: operator.rows[0].CDVENDEDOR}).then(response => {
        if(response && response.rows && response.rows[0]) {
          let [ firstRow ] = response.rows;
          let { CDOPERADOR } = firstRow;
          return CDOPERADOR;
        }
        return null;
      }).catch( error => console.log(error));

      params.P_CDOPERADOR = defaultOperator;
      params.P_UPDATERCONFIGID = configIni.rows[0].ID;

      const appendConfigIni = await updaterModel.appendConfigIniHistory(params);

      if (appendConfigIni.rowsAffected === 0) {
        throw {
          error: true,
          message: 'Falha ao inserir na tabela de histórico de alteração do arquivo config.ini.'
        };
      }

      await OracleService.commit();       
      
      return {
        error: false,
        message: `Config.ini da filial ${params.P_CDFILIAL} foi inserido com sucesso.`
      }
    } catch(error) {     
      await OracleService.rollback();      
      
      return error;
    }     
  }

  async getDefaultOperatorFromSalesman(salesmanCode) {
    try {
      params.P_CDVENDEDOR = salesmanCode;
      const salesman = await updaterModel.getDefaultOperatorFromSalesman(params);

      if (salesman.rows.length === 0 || salesman.rows[0].CDOPERADOR !== null) {
        throw {
          error: true,
          message: `O campo de operador padrão deve estar preenchido ou associado a um vendedor padrão para a filial ${params.P_CDFILIAL}.`
        };
      }

      return salesman.rows[0].CDOPERADOR
    } catch (error) {
      throw error;
    }
  }

  async getUnitConfigIniData(params) {
    try {
      const result = await updaterModel.getConfigIniByUnit(params);
  
      if (result.rows.length === 0) {
        throw new Error(`Não existe um arquivo de configuração salvo para a filial ${params.P_CDFILIAL}`);
      }

      return result.rows[0];
    } catch (error) {
      throw error;
    }
  }

  async getConfigIniByUnit(cdFilial) {
    try {
      const params = {
        P_CDFILIAL: cdFilial,
      };

      const result = await this.getUnitConfigIniData(params);
  
      const decodeConfig = Base64.decode(result.CONFIG);
      
      return {
        error: false,
        message: 'Arquivo de configuração carregado com sucesso!',
        data: decodeConfig
      }
    } catch (error) {
      return {
        error: true,
        message: error.message
      }
    }
  }

  async saveConfigIniByUnit(cdFilial, cdOperador, config) {
    try {
      const encodeConfig = Base64.encode(config);
      let result;

      let params = {
        P_CDFILIAL: cdFilial,
        P_CDOPERADOR: cdOperador,
        P_CONFIG: encodeConfig
      }
      
      result = await updaterModel.updateConfigIniByUnit(params);

      if (result.rowsAffected === 0) {
        throw new Error(`Não existe registro de um arquivo de configuração para a filial ${cdFilial}.`);
      }

      result = await this.getUnitConfigIniData(params);

      params.P_UPDATERCONFIGID = result.ID;

      result = await updaterModel.appendConfigIniHistory(params);
      
      if (result.rowsAffected === 0) {
        throw new Error(`Falha ao salvar o histórico para o arquivo de configuração da filial ${cdFilial}.`);
      }

      await OracleService.commit();
      
      return {
        error: false,
        message: 'Arquivo de configuração salvo com sucesso!'
      }
    } catch (error) {
      await OracleService.rollback();
      
      return {
        error: true,
        message: 'Erro ao salvar o arquivo de configuração. Error: ' + error.message
      }
    }
  }

  async getConfigHistoryByUnit(cdFilial) {
    try {
      let result;
      const params = {
        P_CDFILIAL: cdFilial,
      };

      result = await this.getUnitConfigIniData(params);
      
      params.P_ID = result.ID;
      
      result = await updaterModel.getConfigIniHistoryByUnit(params);
      
      return {
        error: false,
        message: 'Histórico de atualização do arquivo de configuração carregado com sucesso!',
        data: result.rows
      }
    } catch (error) {
      return {
        error: true,
        message: error.message
      }
    }
  }

  handleConfigStructure( configString, configValues = {} ) {
    if(!configString) return {};
    if(Base64.isValid(configString)) {
      return {...Base64.decode(configString), ...configValues};
    }
    return configString.split('|').reduce((acc, key) => {
      acc[key] = (key in configValues) ? configValues[key] : '';
      return acc;
    }, {});
  }

  hasEmptyParams( config ){
    return Object.values(config).some( value => !value);
  }

  setMessageFillEmptyParams( response ){
    if(this.hasEmptyParams(response.data)) {
      response.message = "Preencha os parâmetros do projeto antes de continuar instalação.";
    }
  }

  async getLatestConfig( params ){
    const updaterProdConfigLastUpdate = await updaterModel.getUpdaterProdConfigLastUpdate(params);
    if(updaterProdConfigLastUpdate.rows.length){
      const [ LATEST_VERSION_CONFIG ] = updaterProdConfigLastUpdate.rows;
      const { CONFIG } = LATEST_VERSION_CONFIG;
      return CONFIG;
    }
    return null;
  }

  async getProdConfig(cdProductId, nrVersao) {
    try {
      const params = {
        CDPRODUCTID: cdProductId,
        NRVERSAO: nrVersao
      };

      if (!params.CDPRODUCTID) {
        throw new Error('Codigo do produto não foi informado.'); 
      }
        
      let updaterProdConfig = await updaterModel.getUpdaterProdConfig(params);
      let response = { error: false, data: {} };
      if (updaterProdConfig.rows.length === 0) {
        let CONFIG = await this.getLatestConfig(params);
        await updaterModel.insertUpdaterProdConfig({...params, CONFIG}).catch( error => {
          console.log(error);
          response.error = true;
          response.message = 'Não foi encontrado um registro de configuração para o produto na base de dados.';
          response.errorInfo = error;
        });

        response.data = this.handleConfigStructure(CONFIG);
        if(!response.error) {
          await OracleService.commit();
          this.setMessageFillEmptyParams(response);
        }
        return response;
      }

      let config = updaterProdConfig.rows[0].CONFIG;
      let defaultValues = {};
      let latestConfig = null;
      if(!config) return response;
      if(!Base64.isValid(config)) {
        latestConfig = await this.getLatestConfig(params);
      }
      if(Base64.isValid(latestConfig)) {
        defaultValues = Base64.decode(latestConfig);
      }
      response.data = this.handleConfigStructure(config, defaultValues);
      this.setMessageFillEmptyParams(response);
      return response;
    } catch (error) {
      return {
        error: true,
        message: error.message
      }
    }
  }

  resultNewProductConfiguration(config, decode){
    let prodConfigObject = {};          
    
    config.map((element) => {      
      if (element in decode) {      
        prodConfigObject[element] = decode[element];
      } else {
        prodConfigObject[element] = '';
      }
    })  
    return prodConfigObject;   
  }

  async saveProdConfig(cdProductId, nrVersao, config){
    try {
      for (let item in config) {
        if (config[item] === '') {
          throw new Error('Ocorreu um erro ao salvar as informações, tente novamente.');
        }      
      }

      const encodeProdCondig = Base64.encode(config);

      if (!encodeProdCondig) {
        throw new Error(`Não foi possivel converter os parâmetros do produto ${params.CDPRODUCTID} para base64.`);
      }  
    
      const updaterProductConfig = {
        CDPRODUCTID: cdProductId, 
        NRVERSAO: nrVersao,     
        CONFIG: encodeProdCondig
      }

      const result = await updaterModel.setUpdateProdConfig(updaterProductConfig);
    
      if (result.rowsAffected === 0) {
        throw new Error(`Falha ao salvar parâmetros do produto ${params.CDPRODUCTID}.`);
      }    
    
      const decodeProdCondig = Base64.decode(encodeProdCondig);    

      await OracleService.commit();

      return {
        error: false,
        message: 'Parâmetros do projeto salvo com sucesso!',
        data: decodeProdCondig
      }
    } catch(error) {    
      await OracleService.rollback();
      
      return {
        error: true,
        message: error.message
      }
    }
  }

  async startServiceMonitor(cdFilial, services) {
    try {
      const servicesToAdd = services.map(function(service){
        return service.name
      });

      const params = {
        P_CDFILIAL: cdFilial
      };


      let serviceMonitorConfig = await this.getServiceMonitorConfigurationByUnit(params);
      
      const includeServicesArray = [];

      for (const serviceToAdd of servicesToAdd) {
        if (serviceMonitorConfig.indexOf(serviceToAdd) === -1) {
          includeServicesArray.push(serviceToAdd);
        }
      }

      if (includeServicesArray.length > 0) {
        const regexRemoveSemicolonEndString = new RegExp(';$', 'g');
        serviceMonitorConfig = serviceMonitorConfig.replace(regexRemoveSemicolonEndString, '');
        
        const separator = serviceMonitorConfig !== '' ? ';' : '';

        const includeServices = includeServicesArray.join(';');
        serviceMonitorConfig = `${serviceMonitorConfig}${separator}${includeServices}`;

        params.P_MONITORAMENTO = serviceMonitorConfig;
        
        await updaterModel.updateServiceMonitorConfigurationByUnit(params);
        
        await OracleService.commit();
      }

      return {
        error: false,
        message: 'Monitoramento de serviços atualizado com sucesso!'
      }
    } catch (error) {
      return {
        error: true,
        message: error
      }
    }
  }

  async stopServiceMonitor(cdFilial, services) {
    try {
      const params = {
        P_CDFILIAL: cdFilial
      };

      let serviceMonitorConfig = await this.getServiceMonitorConfigurationByUnit(params);
      
      for (const service of services) {
        const regexService = new RegExp(service, 'g');
        serviceMonitorConfig = serviceMonitorConfig.replace(regexService, '');
      }

      const regexRemoveMultipleSemicolon = new RegExp(';{2,}', 'g');
      serviceMonitorConfig = serviceMonitorConfig.replace(regexRemoveMultipleSemicolon, ';');
      
      const regexRemoveSemicolonEndString = new RegExp(';$', 'g');
      serviceMonitorConfig = serviceMonitorConfig.replace(regexRemoveSemicolonEndString, '');

      params.P_MONITORAMENTO = serviceMonitorConfig;

      await updaterModel.updateServiceMonitorConfigurationByUnit(params);
      
      await OracleService.commit();
      
      return {
        error: false,
        message: 'Monitoramento de serviços removidos com sucesso!'
      }
    } catch (error) {
      return {
        error: true,
        message: error
      }
    }
  }

  async getServiceMonitorConfigurationByUnit(params) {
    try {
      const result = await updaterModel.getServiceMonitorConfigurationByUnit(params);

      if (result.rows.length === 0) {
        throw "Não foi encontrado um monitoramento de serviços ativo.";
      }

      return result.rows[0].MONITORAMENTO ? result.rows[0].MONITORAMENTO : '';
    } catch (error) {
      throw error;
    }
  }

  async installNewVersion(params) {
    try {    
      if(!params.UPDATERPRODID) throw 'Código do produto não informado.';

      const { rows } = await updaterModel.getUpdateLastRecordProduct(params);

      if(rows.length === 0) throw 'Erro ao efetuar busca do produto.';
     
      if(rows[0].NRVERSAO === params.NRVERSAO) throw 'Essa versão já está instalada.';

      const newVersionObject = {       
        UPDATERPRODID: params.UPDATERPRODID,
        NRVERSAO: params.NRVERSAO,
        DSURL: params.DSURL,
        DSDIRETORIO: rows[0].DSDIRETORIO,        
        IDATIVO: rows[0].IDATIVO,    
        IDDOWNLOADVER: 'N',
        IDAPROVVER: rows[0].IDAPROVVER
      };     

      const { rowsAffected } = await updaterModel.setUpdateProdVersion(newVersionObject);

      if(rowsAffected === 0) throw 'Erro ao inserir informações do produto.';  

      const lastConfig = await this.getLatestConfig({CDPRODUCTID: params.UPDATERPRODID, NRVERSAO: params.NRVERSAO});

      const prodConfig = {
        CDPRODUCTID: params.UPDATERPRODID, 
        NRVERSAO: params.NRVERSAO, 
        CONFIG: lastConfig       
      };

      const resultSet = await updaterModel.insertUpdaterProdConfig(prodConfig);
      
      if(resultSet.rowsAffected === 0) throw 'Erro ao inserir informações do produto.';  

      await OracleService.commit();

      return {
        error: false,
        message: 'As informações foram salvas com sucesso.'
      }
    } catch (error) {
      await OracleService.rollback();
      return {
        error: true,
        message: error
      }    
    }
}

  async removeEventsByUnitAndProduct( params ) {
    const response = {
      error: false,
      message: ''
    }
    try {
      await updaterModel.removeEventsByUnitAndProduct(params).catch( error => {
        response.error = true;
        response.message = error.message;
      });
    } catch( error ) {
      response.error = true;
      response.message = error.message;
    }
    return response;
  }
}

module.exports = new UpdaterService();
