import Prism from 'prismjs';
import { Http, IEventParam, Metadata, Config } from '@zeedhi/core';
import { TekGrid } from '@zeedhi/teknisa-components-common';
import { AlertService, DialogService, Form, LoadingService, Modal, ModalService, Select, Text, TextInput, Button, IterableComponentRender } from '@zeedhi/common';
import { Highlight } from '@zeedhi/zd-highlight-common';
import { io, Socket } from 'socket.io-client';
import { SocketEvent } from '@/socket/socket-event';
import { ConfigIni } from '../interfaces/configIni';
import 'prismjs/components/prism-markup-templating.min';
import 'prismjs/components/prism-sql.min';

export class ParametrizacoesController {
  private paramEditModal!: Modal;

  private paramFilialEditModal!: Modal;

  private socketArgs = {
    email: 'assistant@teknisa.com',
    password: 'nineninenine',
    slave: false,
  };

  public isVisibleRowHighlight = true;

  public socket?: Socket;

  private dataSourceEdit = [{
      value: 'I',
      text: 'Assistant (Proprietário)',
  }, {
      value: 'A',
      text: 'Base de Dados - Cloud',
  }, {
      value: 'L',
      text: 'Base de Dados - Loja',
  }, {
      value: 'C',
      text: 'Apache - Cloud',
  }, {
      value: 'E',
      text: 'Apache - Loja',
  }, {
      value: 'S',
      text: 'Serviços - Loja',
  }];

  private dataSourceInsert = [{
      value: 'A',
      text: 'Base de Dados - Cloud',
  }, {
      value: 'L',
      text: 'Base de Dados - Loja',
  }, {
      value: 'C',
      text: 'Apache - Cloud',
  }, {
      value: 'E',
      text: 'Apache - Loja',
  }, {
      value: 'S',
      text: 'Serviços - Loja',
  }];

  public openFilter({ component }: any) {
    const filterButton = Metadata.getInstance<Button>(`${component.name}_filterButton`);
    filterButton.click({} as Event, {} as HTMLElement);
  }

  public async beforeApplyFilter({ component }: any) {
    if (!this.socket) {
      this.socket = io(Config.endPoint, {
        transports: ['websocket', 'polling'],
      });
    }
    this.socket?.emit(SocketEvent.CONNECTION);
    this.socket?.emit('login', this.socketArgs);

    const dataFilter = component.getFilterInputs('typePmz')[0].value;
    const gridParams = component.parent.getChildInstance('gridParams');
    const gridParamFilial = component.parent.getChildInstance('gridParamFilial');
    component.isVisible = false;

    if (dataFilter === '1') {
      gridParams.isVisible = true;
      this.setFilterPopup(dataFilter, gridParams.datasource);
      gridParams.reload();
      if (!this.paramEditModal) this.paramEditModal = await ModalService.createFromJson('parametrizacoes-edit', true);
    } else {
      gridParamFilial.isVisible = true;
      this.setFilterPopup(dataFilter, gridParamFilial.datasource);
      gridParamFilial.reload();
    }
  }

  private setFilterPopup(filter:string, datasource: any) {
    if (!datasource.dynamicFilter.typePmz) {
      datasource.dynamicFilter = {
        typePmz: [{
          operation: 'CONTAINS',
          relation: 'AND',
          value: filter,
        }],
      };
    } else if (datasource.dynamicFilter.typePmz[0].value !== filter) {
        datasource.dynamicFilter.typePmz[0].value = filter;
     }
  }

	public gridBeforeDestroy() {
		this.paramEditModal?.destroy();
    this.paramFilialEditModal?.destroy();
    this.socket?.removeAllListeners('updateConfigInit');
    this.socket?.removeAllListeners('updateConfigIniFinish');
	}

  public loadLanguage() {
    const { code, language } = Metadata.getInstance<Highlight>('COMANDO');
    return Prism.highlight(code || '', Prism.languages[language], language);
  }

	public async gridCellClick({ component, row } : any) {
		if (!component.editing) {
			await this.paramEditModal.show();
			const form = Metadata.getInstance<Form>('edit-modal-form');
      const typeColumn: Select = form.getChildInstance('TIPO');
      const nameColumn: TextInput = form.getChildInstance('NOME');
      const commandObsColumn: Text = form.getChildInstance('COMMAND_OBS');
      form.value = row;
      if (row.TIPO === 'I') {
        this.isVisibleRowHighlight = false;
        typeColumn.datasource.updateData(this.dataSourceEdit);
        typeColumn.readonly = true;
        nameColumn.readonly = true;
        commandObsColumn.text = 'Comandos internos não podem ser alterados';
      } else {
        typeColumn.datasource.updateData(this.dataSourceInsert);
        typeColumn.readonly = false;
        nameColumn.readonly = false;
        this.changeModalFormFields(row.TIPO);
      }
		}
	}

  public capitalizeFirstLetter(string: String) {
    return string.charAt(0).toUpperCase() + string.slice(1);
  }

  public async buttonConfigClick({ row } : any) {
    LoadingService.show();
    this.paramFilialEditModal?.destroy();
    this.paramFilialEditModal = await ModalService.createFromJson('parametrizacoes-filial-configini', true);
    await this.paramFilialEditModal.show();

    const params = { CDFILIAL: row.CDFILIAL };
    const modal = this.paramFilialEditModal;
    const setCdfilial: any = modal.getChildInstance('CDFILIAL');
    setCdfilial.text = row.CDFILIAL;

    Http.post('/updater/getConfigIniByUnit', params).then((result) => {
      if (result.data.error) {
        throw result.data.message;
      }

      const configInterface : ConfigIni = result.data.data;
      const card: any = modal.getChildInstance('collapseCardIterable');
      const titleHeader = Metadata.getInstance<Text>('view-modal-title-filial');
      titleHeader.text += ` Filial ${row.CDFILIAL} - ${row.NMFILIAL}`;

      if (Object.keys(configInterface).length > 0) {
        Object.entries(configInterface).forEach(([grupo, data]) => {
          const obj: any = {
            id: `collapseCard-${grupo}`,
            name: this.capitalizeFirstLetter(grupo),
            list: [],
          };

          Object.entries(data).forEach(([prop, valor]) => {
            obj.list.push({
              id: `field-${prop}`,
              name: prop,
              value: valor,
              label: prop,
            });
          });

          card.datasource.data.push(obj);
        });
      }
    }).catch((error) => {
      this.showAlert(error, 'error');
    }).finally(() => {
      LoadingService.hide();
    });
	}

  public modalConfig({ component } : IEventParam<IterableComponentRender>) {
    const parentIterable = Metadata.getInstance<IterableComponentRender>('collapseCardIterable');
    const id = component.name.split('_').pop();
    const parentIndex = parentIterable.datasource.data.findIndex((row) => row.id === id);
    component.datasource.data = parentIterable.datasource.data[parentIndex].list;
  }

  public closeParamFilialEditModal() {
		this.paramFilialEditModal.hide();
    this.paramFilialEditModal.destroy();
	}

  public async saveConfigIni() {
    try {
      const card: any = this.paramFilialEditModal.getChildInstance('collapseCardIterable');
      const cdFilial: any = this.paramFilialEditModal.getChildInstance('CDFILIAL');
      const dados: any = {};

      if (card.datasource.data.length === 0) {
        throw new Error('Não há dados para salvar');
      }

      LoadingService.show();

      card.datasource.data.forEach((group:any) => {
        const form: any = Metadata.getInstance<Form>(`formCard-${group.name}`);

        if (!form.validate()) {
          throw new Error('Falha na validação do formulário. Favor preencher corretamente todos os campos.');
        }

        const inputs = [...form.element].filter((item) => item.type === 'text');
        const obj: any = {};

        inputs.forEach((row: any) => {
          const key = row.name.split('-').pop();
          obj[key] = row.value;
        });

        dados[group.name.toLowerCase()] = obj;
      });

      const params = {
        CDFILIAL: cdFilial.text,
        CDOPERADOR: localStorage.getItem('CDOPERADOR'),
        CONFIG: dados,
      };

      const { data } = await Http.post('/updater/saveConfigIniByUnit', params);

      if (data.error) {
        throw new Error(data.message);
      }

      this.closeParamFilialEditModal();
      this.socket?.emit('updateConfigInit', cdFilial.text);
      this.resultUpdaterConfigFili();
    } catch (error) {
      if (error instanceof Error) {
        this.showAlert(error.message, 'error');
      }
    } finally {
      LoadingService.hide();
    }
  }

  private resultUpdaterConfigFili(): void {
    this.socket?.on('updateConfigIniFinish', (params) => {
      const color = params.error ? '#e68129' : 'success';

      this.showAlert(params.message, color);
    });
  }

  public async buttonHistoryClick({ row }: any) {
    try {
      LoadingService.show();

      this.paramFilialEditModal?.destroy();
      this.paramFilialEditModal = await ModalService.createFromJson('parametrizacoes-filial-history', true);

      const params = { CDFILIAL: row.CDFILIAL };

      const { data } = await Http.post('/updater/getConfigHistoryByUnit', params);

      if (data.error) {
        throw new Error(data.message);
      }

      const grid = this.paramFilialEditModal.children[1];
      grid.datasource.data = data.data;
      await this.paramFilialEditModal.show();
      const titleHeader = Metadata.getInstance<Text>('history-modal-title-filial');
      titleHeader.text += ` Filial ${row.CDFILIAL} - ${row.NMFILIAL}`;
    } catch (error) {
      this.showAlert(error, 'error');
    } finally {
      LoadingService.hide();
    }
  }

  public changeModalTipo(params: any) {
    this.changeModalFormFields(params.value);
  }

  private changeModalFormFields(type: any) {
    const form = Metadata.getInstance<Form>('edit-modal-form');
    const commandApacheColumn: TextInput = form.getChildInstance('COMANDO_APACHE');
    const commandServicesColumn: TextInput = form.getChildInstance('COMANDO_SERVICOS');
    const commandObsColumn: Text = form.getChildInstance('COMMAND_OBS');

    commandApacheColumn.removeValidation('required');
    commandServicesColumn.removeValidation('required');

    if (type === 'C' || type === 'E') {
      this.isVisibleRowHighlight = false;
      commandServicesColumn.isVisible = false;
      commandApacheColumn.isVisible = true;
      commandApacheColumn.addValidation('required', {});
      form.value.COMANDO_APACHE = form.value.COMANDO;
      commandObsColumn.text = 'Esse monitoramento será considerado como sucesso quando a requisição GET feita para os endereços informados funcionar.';
    } else if (type === 'S') {
      this.isVisibleRowHighlight = false;
      commandApacheColumn.isVisible = false;
      commandServicesColumn.isVisible = true;
      commandServicesColumn.addValidation('required', {});
      form.value.COMANDO_SERVICOS = form.value.COMANDO;
      commandObsColumn.text = 'Esse monitoramento será considerado como sucesso quando todos os serviços informados estiverem em execução.';
    } else if (type !== 'I') {
      this.isVisibleRowHighlight = true;
      commandServicesColumn.isVisible = false;
      commandApacheColumn.isVisible = false;
      const commandComponent = Metadata.getInstance<Highlight>('COMANDO');
      commandComponent.code = form.value.COMANDO;
      commandObsColumn.text = 'Esse monitoramento será considerado como sucesso quando a consulta não retornar nenhum registro.';
    }
  }

	public closeParamEditModal() {
		this.paramEditModal.hide();
	}

  public async addClick() {
    await this.paramEditModal.show();
    const form = Metadata.getInstance<Form>('edit-modal-form');
    const typeColumn: Select = form.getChildInstance('TIPO');
    typeColumn.datasource.updateData(this.dataSourceInsert);
    typeColumn.readonly = false;
    form.value.TIPO = 'A';
  }

  public getConditionalCssClass(param: any) {
    return param.row.ATIVO === 'N' ? 'ass-inactive' : '';
  }

  public async updateInModal() {
    const grid = Metadata.getInstance<TekGrid>('gridParams');
    const form = Metadata.getInstance<Form>('edit-modal-form');
    const commandComponent = Metadata.getInstance<Highlight>('COMANDO');
    if (!form.validate()) return;
    if (!commandComponent.code && (form.value.TIPO === 'A' || form.value.TIPO === 'L')) {
      this.showAlert('Informe o comando', 'warning');
      return;
    }

    if (form.value.TIPO === 'I') {
      form.value.COMANDO = 'INTERNO';
    } else if (form.value.TIPO === 'A' || form.value.TIPO === 'L') {
      form.value.COMANDO = commandComponent.code;
    } else if (form.value.TIPO === 'C' || form.value.TIPO === 'E') {
      form.value.COMANDO = form.value.COMANDO_APACHE;
    } else {
      form.value.COMANDO = form.value.COMANDO_SERVICOS;
    }
    const uri = form.value.ID ? `/data/connect/portal/${form.value.ID}` : '/data/insertAssConfig';
    const {
      ID,
      NOME,
      TIPO,
      COMANDO,
      INTERVALO,
    } = form.value;
    LoadingService.show();
    Http.put(uri, form.value).then(() => {
      this.socket?.emit('updateMonitoringParams', { ID, NOME, TIPO, COMANDO, INTERVALO });
      LoadingService.hide();
      this.paramEditModal.hide();
      grid.datasource.get();
      const message = form.value.ID ? 'Registro atualizado com sucesso!' : 'Registro inserido com sucesso!';
      this.showAlert(message, 'success');
    }).catch((error) => {
      LoadingService.hide();
      this.showAlert(error, 'error');
    });
	}

  public async beforeDelete(params: any) {
    params.event.preventDefault();
    const grid = Metadata.getInstance<TekGrid>('gridParams');

    DialogService.show({
			name: `confirm_${grid.name}`,
			type: 'warning',
			title: 'Exclusão',
			text: 'Deseja realmente excluir os registros selecionados?',
			buttons: [
				{
          name: 'no',
          component: 'ZdButton',
          label: 'Não',
          outline: true,
          events: {
            click: () => DialogService.hide(),
          },
        },
				{
          name: 'yes',
          component: 'ZdButton',
          label: 'Sim',
          events: this.deleteConfig,
        },
			],
		});
  }

  private deleteConfig: any = {
    click: () => this.doBeforeDelete(),
  };

  private async doBeforeDelete() {
    const grid = Metadata.getInstance<TekGrid>('gridParams');

    const ids: any = [];
    grid.selectedRows.forEach((element) => {
      if (element.ID) {
        ids.push(element.ID);
      }
    });

    const rq = `connect/portal/${ids.join(',')}`;
    const conexao = process.env.VUE_APP_METADATA_END_POINT + rq;

    LoadingService.show();
    Http.delete(conexao).then(() => {
      this.paramEditModal.hide();
      grid.datasource.get();
      const message = 'Registros removidos com sucesso!';
      grid.selectedRows = [];
      this.showAlert(message, 'success');
    }).catch((error) => {
      this.showAlert(error, 'error');
    }).finally(() => {
      LoadingService.hide();
      DialogService.hide();
    });
  }

  private showAlert(message: string | any, colorStr: string) {
    AlertService.show({
      name: 'info-alert',
      text: message,
      color: colorStr,
      right: true,
      top: true,
      showDismiss: true,
      timeout: 5000,
    });
  }
}
